;;; FILE: "eg.scheme"
;;; IMPLEMENTS: Framework for Integrated Test
;;; AUTHOR: Ken Dickey
;;;  See http://fit.c2.com for documentation and examples

;;;
;;; Copyright (c) 2002 Cunningham & Cunningham, Inc.
;;; Released under the terms of the GNU General Public License version 2 or later.
;;;

(define pi 3.14159265358979323846)

(define (degrees->radians degrees)
  (/ (* degrees pi) 180))

(define (radians->degrees radians)
  (/ (* radians 180) pi))


;;;==============================================
;;; class  eg.ArithmeticFixture
;;;==============================================
(define-class eg.ArithmeticFixture (<fixture>) 
  ( (x :accessor x :init-form #f )
    (y :accessor y :init-form #f ) )
)

(define-method type-map ( (f eg.ArithmeticFixture) (label <string>) )
  'number ;; everything is a number
;   (cond
;    ((equal? label "/") 'number)
;    ((equal? label "y") 'number)
;    (else (next-method))
; )
)

(define-method do-rows ( (f eg.ArithmeticFixture) (rows <parse>) )
  (let loop ( (row (more rows)) ) ;; skip the 1st row
    (when row
	  (do-row f row)
	  (loop (more row)))
) )


(define-method do-cell ( (f eg.ArithmeticFixture) 
			 (cell <parse>) 
			 (column-number <integer>) )
  (when *debug-fit*
	(format #t "cell=~a~%" (text cell)))
  (case column-number
    ((0) (set! (x f) (read-number-from-cell cell)))
    ((1) (set! (y f) (read-number-from-cell cell)))
    ((2) (check f cell (+ (x f) (y f))))
    ((3) (check f cell (- (x f) (y f))))
    ((4) (check f cell (* (x f) (y f))))
    ((5) (check f cell (/ (x f) (y f))))
    (else (ignore f cell)))
)


 
;;;==============================================
;;; class eg.ArithmeticColumnFixture
;;;==============================================

(define-class eg.ArithmeticColumnFixture (<column-fixture>) 
  ( (x :accessor x :init-form #f )
    (y :accessor y :init-form #f ) )
)

(define-method plus ( (f eg.ArithmeticColumnFixture) )
  (+ (x f) (y f)))

(define-method minus ( (f eg.ArithmeticColumnFixture) )
  (- (x f) (y f)))

(define-method times ( (f eg.ArithmeticColumnFixture) )
  (* (x f) (y f)))

(define-method divide ( (f eg.ArithmeticColumnFixture) )
;  (format t "~%divide x=~s, y=~s, x/y=~s~%" (x f) (y f) (/ (x f) (y f))) ;;@@@
  (/ (x f) (y f)))

(define-method floating ( (f eg.ArithmeticColumnFixture) )
  (+ 0.0 (/ (x f) (y f))))

(define-method type-map ( (f eg.ArithmeticColumnFixture) (label <string>) )
  (cond
   ((equal? label "floating") 'float)
   (else (next-method))
) )

;;;==============================================
;;; class <hp35-calculator>
;;;==============================================

(define-class <hp35-calculator> () 
  ( (registers :accessor registers 
	       :init-form (vector 0 0 0 0))
    (store :accessor store :init-form 0)
  )
)


(define-method key ( (calculator <hp35-calculator>) (key <string>) )
  (when *debug-fit*
	(format #t "key ~s~%" key))
  (cond
   ((key-numeric? key)
    (do-push calculator (number-from-string key))
   )
   ((equal? key "enter")
    (raise-stack calculator)
   )
   ((equal? key "+")
    (do-push calculator (+ (do-pop calculator) (do-pop calculator)))
   )
   ((equal? key "-")
    (do-push calculator 
       (let* ( (x (do-pop calculator))
	       (y (do-pop calculator))
	     )
	 (- y x)))
   )
   ((equal? key "*")
    (do-push calculator (* (do-pop calculator) (do-pop calculator)))
   )
   ((equal? key "/")
    (do-push calculator 
       (let* ( (x (do-pop calculator))
	       (y (do-pop calculator))
	     )
;	 (format #t "x=~a, y=~a, y/x=~a~%" x y (/ y x))
	 (/ y x)))
   )
   ((equal? key "x^y")
    (do-push  calculator 
       (let* ( (x (do-pop calculator))
	       (y (do-pop calculator))
	     )
	 (expt x y)))
   )
   ((equal? key "clx")
    (let ( (regs (registers calculator)) )
      (set! (vector-ref regs 0) 0)
   ))
   ((equal? key "clr")
    (set! (registers calculator) (vector 0 0 0 0))
   )
   ((equal? key "chs")
    (let ( (regs (registers calculator)) )
     (set! (vector-ref regs 0) (- (vector-ref regs 0)))
   ))
   ((equal? key "x<>y")
    (let ( (regs (registers calculator)) )
      (let ( (reg0 (vector-ref regs 0))
	     (reg1 (vector-ref regs 1))
	   )
	(set! (vector-ref regs 0) reg1)
	(set! (vector-ref regs 1) reg0)
   )))
   ((equal? key "r!")
    (let ( (regs (registers calculator)) )
      (set! (vector-ref regs 3) (do-pop calculator))
   ))
   ((equal? key "sto")
    (set! (store calculator) (vector-ref (registers calculator) 0))
   )
   ((equal? key "rcl")
    (do-push calculator (store calculator))
   )
   ((equal? key "sqrt")
    (do-push calculator (sqrt (do-pop calculator)))
   )
   ((equal? key "ln")
    (do-push calculator (log (do-pop calculator)))
   )
   ((equal? key "sin")
    (do-push calculator (sin (degrees->radians (do-pop calculator))))
   )
   ((equal? key "cos")
    (do-push calculator (cos (degrees->radians (do-pop calculator))))
   )
   ((equal? key "tan")
    (do-push calculator (tan (degrees->radians (do-pop calculator))))
   )
   (else (error (format #f "calculator: can't do key: ~s" key)))
) )

(define-method do-push ( (c <hp35-calculator>) (value <number>) )
  (raise-stack c)
  (set! (vector-ref (registers c) 0) value)
)

(define-method raise-stack ( (c <hp35-calculator>) )
  (let ( (regs (registers c)) )
    (set! (vector-ref regs 3) (vector-ref regs 2))
    (set! (vector-ref regs 2) (vector-ref regs 1))
    (set! (vector-ref regs 1) (vector-ref regs 0))
) )

(define-method do-pop ( (c <hp35-calculator>) )
  (let* ( (regs (registers c)) 
	  (result (vector-ref regs 0))
        )
    (set! (vector-ref regs 0) (vector-ref regs 1))
    (set! (vector-ref regs 1) (vector-ref regs 2))
    (set! (vector-ref regs 2) (vector-ref regs 3))
    result
) )

(define-method key-numeric? ( (key <string>) )
  (number? (call-with-input-string key (lambda (p) (read p))))
)

(define-method number-from-string ( (key <string>) )
  (let ( (value (call-with-input-string key (lambda (p) (read p)))) )
    (if (number? value)
	value
        (error (format #f "number-from-string: not a number: ~s" key)))
) )



;;;==============================================
;;; class  eg.Calculator
;;;==============================================

(define-class eg.Calculator (<column-fixture>)
  ( (volts :accessor volt  :init-form 0)
    (a-key :accessor a-key :init-form '"")
    (hp    :accessor hp    :init-form (make <hp35-calculator>))
  )
)

(define-method label-map ( (f eg.Calculator) (label <string>) )
  (if (equal? label "key")
      "a-key"
      label)
)

(define-method type-map ( (f eg.Calculator) (label <string>) )
  (cond
   ((string-ci=? label "points") 'boolean)
   ((string-ci=? label "flash")  'boolean)
   ((string-ci=? label "key")    'string)
   ((string-ci=? label "x")      'number)
   ((string-ci=? label "y")      'number)
   (else (next-method))
) )

(define-method points ( (fixture eg.Calculator) )
  #f)

(define-method flash ( (fixture eg.Calculator) )
  #f)

(define-method watts ( (fixture eg.Calculator) )
  0.5)

(define-method reset ( (fixture eg.Calculator) )
  (set! (key fixture) #f))

(define-method do-execute ( (fixture eg.Calculator) )
  (and (a-key fixture)
       (key (hp fixture) (a-key fixture))
) )

(define-method x ( (fixture eg.Calculator) )
  (do-execute fixture)
  (vector-ref (registers (hp fixture)) 0)
)

(define-method y ( (fixture eg.Calculator) )
  (vector-ref (registers (hp fixture)) 1)
)

(define-method z ( (fixture eg.Calculator) )
  (vector-ref (registers (hp fixture)) 2)
)

(define-method t ( (fixture eg.Calculator) )
  (vector-ref (registers (hp fixture)) 3)
)

(define-method points ( (fixture eg.Calculator) )
  #f)


;;; --- E O F --- ;;;
