;;; FILE: "fit.lisp"
;;; IMPLEMENTS: Framework for Integrated Test
;;;  See http://fit.c2.com for documentation and examples
;;;
;;; Copyright (C) 2002 by Ken Dickey & Ward Cunningham
;;;  This is Free Software, covered by the GNU GPL (v2)
;;;  See http://www.gnu.org/copyleft/gpl.html
;;;
;;; NOTES:
;;;  The code follows the structure of the Java FIT tests

;(eval-when (compile load eval)
;;  @@FIXME (require :cl-unit-test)
;  (require :cllib-html    (translate-logical-pathname "cllib:html"))
;  (require :cllib-htmlgen (translate-logical-pathname "cllib:htmlgen"))
;)

(in-package :user)

;(export
; '())

(delete-all-tests) ;; remove unit-test tests before adding our own

(defmacro test-assert-equal (expected actual)
  (let ( (test-name (format nil "~a" `,expected)) )
    `(deftest ,test-name ,test-name 
       (test-assert (equal ,expected ,actual)))))


;;============================================

;;(deftest :testParsing "the parse (1)"
  (setq p (do-parse "leader<Table foo=2>body</table>trailer" '("table")));)

; (deftest :testParsing "leader"
;   (test-assert (string-equal (leader p) "leader")))

(test-assert-equal  "leader" (leader p))

; (deftest :testParsing "start-tag"
;   (test-assert (string-equal (start-tag p) "<Table foo=2>")))
(test-assert-equal  "<Table foo=2>" (start-tag p))

; (deftest :testParsing "body"
;   (test-assert (string-equal (body p) "body")))
(test-assert-equal  "body" (body p))

; (deftest :testParsing "trailer"
;   (test-assert (string-equal (trailer p) "trailer")))
(test-assert-equal  "trailer" (trailer p))

;;================================================

;;(deftest :testRecursing "the parse (2)"
  (setq q (do-parse "leader<table><TR><Td>body</tD></TR></table>trailer" 
		    '("table" "tr" "td")));)

; (deftest :testRecursing "table body"
;   (test-assert (string-equal (body q) nil)))
(test-assert-equal nil (body q))


; (deftest :testRecursing "row body"
;   (test-assert (string-equal (body (parts q)) nil)))
(test-assert-equal nil (body (parts q)))


; (deftest :testRecursing "data body"
;   (test-assert (string-equal (body (parts (parts q))) "body")))
(test-assert-equal "body" (body (parts (parts q))))

;;================================================

;;(deftest :testIterating "the parse (3)"
  (setq r (do-parse "leader<table><tr><td>one</td><td>two</td><td>three</td></tr></table>trailer"
		    '("table" "tr" "td")));)

; (deftest :testIterating "data body one"
;   (test-assert (string-equal (body (parts (parts r))) "one")))
(test-assert-equal "one"  (body (parts (parts r))))

; (deftest :testIterating "data body two"
;   (test-assert (string-equal (body (more (parts (parts r)))) "two")))
(test-assert-equal "two"  (body (more (parts (parts r)))))

; (deftest :testIterating "data body three"
;   (test-assert (string-equal (body (more (more (parts (parts r))))) "three")))
(test-assert-equal "three"  (body (more (more (parts (parts r))))))

;;===============================================

(setq tags '("td"))
(setq zz (do-parse "<td>a&lt;b</td>" tags))
(test-assert-equal "a&lt;b" (body zz))
(test-assert-equal "a<b"    (text zz))
(setq z1 (do-parse "<td>	a&gt;b&nbsp;&amp;&nbsp;b>c &&&nbsp;</td>" tags))
(test-assert-equal "a>b & b>c &&"  (text z1))
(setq z2 (do-parse 
	  "<TD><P><FONT FACE=\"Arial\" SIZE=2>GroupTestFixture</FONT></TD>"
	  tags))
(test-assert-equal "GroupTestFixture" (text z2))
(test-assert-equal "a<b" 
		   (unescape "a&lt;b"))
(test-assert-equal "a>b & b>c &&" 
		   (unescape "a&gt;b&nbsp;&amp;&nbsp;b>c &&"))
(test-assert-equal "ab" 
		   (unformat "<font size=+1>a</font>b"))
(test-assert-equal "ab"  
		   (unformat "a<font size=+1>b</font>"))
(test-assert-equal "a<b" 
		   (unformat "a<b"))

(setq p0 (do-parse "<td>	a&gt;b&nbsp;&amp;&nbsp;b>c &&nbsp;</td>" tags))
(test-assert-equal "a>b & b>c &" (text p0))
(test-assert-equal "&amp;&amp;" (unescape "&amp;amp;&amp;amp;"))



;;===============================================


;;(deftest :testIndexing "the parse (4)"
  (setq s (do-parse 
	  "Leader<table><tr><td>one</td><td>two</td><td>three</td></tr><tr><td>four</td></tr></table>trailer"
	   '("table" "tr" "td")));)

(test-assert-equal "one"   (body (at3 s 0 0 0)))
(test-assert-equal "two"   (body (at3 s 0 0 1)))
(test-assert-equal "three" (body (at3 s 0 0 2)))
(test-assert-equal "three" (body (at3 s 0 0 3)))
(test-assert-equal "three" (body (at3 s 0 0 4)))
(test-assert-equal "four"  (body (at3 s 0 1 0)))
(test-assert-equal "four"  (body (at3 s 0 1 1)))
(test-assert-equal "four"  (body (at3 s 0 2 0)))
(test-assert-equal 1       (size s))
(test-assert-equal 2       (size (parts s)))
(test-assert-equal 3       (size (parts (parts s))))
(test-assert-equal "one"   (body (leaf s)))
(test-assert-equal "four"  (body (leaf (last-of (parts s)))))

(defun test-html-page ( base-file-name )
  (let* ( (in-name  (concatenate 'string +root-path+ "/Release/Documents/" base-file-name))
	  (out-name (concatenate 'string "/tmp/" base-file-name))
	  (tables   (do-parse (read-file-into-string in-name) 
			      '("table" "tr" "td")))
	  (fixture  (make-instance '<fixture>))
        )
    (reset-counts fixture)
    (do-tables fixture tables)
    (with-open-file (outp out-name :direction :output)
		    (print-to-stream tables outp))
    (format t "~%    ==>> Check file ~a for results" out-name)
) )

(defun test-wiki-page ( base-file-name )
  (format t "~%==>>processing ~s~%" base-file-name)
  (let* ( (in-name  (concatenate 'string +root-path+ "/Release/Documents/" base-file-name))
	  (out-name (concatenate 'string "/tmp/" base-file-name))
	  (tables   (do-parse (read-file-into-string in-name) 
			      '("wiki" "table" "tr" "td")))
	  (fixture  (make-instance '<fixture>))
        )
    (reset-counts fixture)
    (do-tables fixture (parts tables))
    (with-open-file (outp out-name :direction :output)
		    (print-to-stream tables outp))
    (format t "~%    ==>> Check file ~a for results" out-name)
) )

;;=======================================================
;; Arithmetic test

;; +root-path+ is defined in file "load-me.lisp"

(deftest :testFile1 "the parse (6)"
   (test-html-page "arithmetic.html"))

;;=======================================================
;; Calculator test

(deftest :testFile2 "the parse (7)"
  (test-wiki-page "CalculatorExample.html"))

;;=======================================================
;; Music test

(deftest :testFile3 "the parse (8)"
  (test-wiki-page "MusicExample.html"))

(deftest :testFile4 "the parse (9)"
  (test-wiki-page "MusicExampleWithErrors.html"))

;;; --- E O F --- ;;;
