;;;;   -*- Mode: lisp; Package: common-lisp-user; Syntax: Common-lisp -*-
;;
;; 
;; Author:  @@@ I believe that much of this is cribbed from various
;;	    @@@ Lisp books, but I don't have them to check.  Alain?
;; Version: $Id: generic-macros.lisp,v 1.1 2002/10/15 05:11:05 kend Exp $
;;
;;;; Commentary:
;;	Common macros used by but not directly related to
;;	project code.  [Reusable].
;; 
;; 
;;;; Code:

(defconstant +generic-macros-version+ "$Revision"
  "$Id: generic-macros.lisp,v 1.1 2002/10/15 05:11:05 kend Exp $

Report bugs to: bugs@memetrics.com")

;;(in-package bullseye-package)
(in-package :common-lisp-user)



(defun one-of (&rest objects)
  "Return one of the objects, selected at random."
  (let ((len (length objects)))
    (nth (random len) objects)))

(defmacro for-each (fn seq)
  `(progn (map nil ,fn ,seq) t))

(defmacro while (test &body body)
  `(do ()
       ((not ,test))
     ,@body))

(defmacro until (test &body body)
  `(do ()
       (,test)
     ,@body))

(defun circular (lst)
  (setf (cdr (last lst)) lst))

(defmacro bind-if ((var pred) then &optional else)
  `(let ((,var ,pred))
     (if ,var ,then ,else)))

(defmacro if-it (pred then &optional else)
  (let ((it (gensym "IT")))
    `(let ((,it ,pred))
      (symbol-macrolet ((it ,it))
	(if ,it ,then ,else)))))

(defmacro bind-when ((var pred) &body body)
  `(let ((,var ,pred))
     (when ,var ,@body)))

(defmacro when-it (test-form &body body)
  `(if-it ,test-form
        (progn ,@body)))

(defmacro while-it (expr &body body)
  (let ((it (gensym "IT")))
    `(let (,it)
      (loop
       for ,it = ,expr
       while ,it
       do (macrolet ((it ,it))
	    ,@body)))))

(defmacro and-it (&rest args)
  (cond ((null args) t)
        ((null (cdr args)) (car args))
        (t `(if-it ,(car args) (and-it ,@(cdr args))))))

;; FIXME: non-hygenic!
(defmacro cond-it (&rest clauses)
  (if (null clauses)
      nil
      (let ((cl1 (car clauses))
            (sym (gensym)))
        `(let ((,sym ,(car cl1)))
           (if ,sym
               (let ((it ,sym)) ,@(cdr cl1))
               (cond-it ,@(cdr clauses)))))))

(defmacro alambda (parms &body body)
  `(labels ((self ,parms ,@body))
     #'self))

(defmacro ablock (tag &rest args)
  `(block ,tag
     ,(funcall (alambda (args)
                 (case (length args)
                   (0 nil)
                   (1 (car args))
                   (t `(let ((it ,(car args)))
                         ,(self (cdr args))))))
               args)))

(defmacro if-it2 (test &optional then else)
  (let ((win (gensym))
	(it (gensym "IT")))
    `(multiple-value-bind (,it ,win) ,test
      (macrolet ((it ,it))
       (if (or ,it ,win) ,then ,else)))))
;; (defmacro if-it2 (test &optional then else)
;;   (let ((win (gensym)))
;;     `(multiple-value-bind (it ,win) ,test
;;        (if (or it ,win) ,then ,else))))

(defmacro when-it2 (test &body body)
  `(if-it2 ,test
         (progn ,@body)))

(defmacro while-it2 (test &body body)
  (let ((flag (gensym)))
    `(let ((,flag t))
       (while ,flag
         (if-it2 ,test
               (progn ,@body)
               (setq ,flag nil))))))

;; FIXME: non-hygenic!
(defmacro cond-it2 (&rest clauses)
  (if (null clauses)
      nil
      (let ((cl1 (car clauses))
            (val (gensym))
            (win (gensym)))
        `(multiple-value-bind (,val ,win) ,(car cl1)
           (if (or ,val ,win)
               (let ((it ,val)) ,@(cdr cl1))
               (cond-it2 ,@(cdr clauses)))))))

(defmacro with-places (pat seq &body body)
  (let ((gseq (gensym)))
    `(let ((,gseq ,seq))
       ,(wplac-ex (destruc pat gseq #'atom) body))))

(defun wplac-ex (binds body)
  (if (null binds)
      `(progn ,@body)
      `(symbol-macrolet ,(mapcar #'(lambda (b)
                                     (if (consp (car b))
                                         (car b)
                                         b))
                                 binds)
        ,(wplac-ex (mapcan #'(lambda (b)
                               (if (consp (car b))
                                   (cdr b)))
                           binds)
                   body))))

(defun destruc (pat seq &optional (atom? #'atom) (n 0))
  (if (null pat)
      nil
      (let ((rest (cond ((funcall atom? pat) pat)
                        ((eq (car pat) '&rest) (cadr pat))
                        ((eq (car pat) '&body) (cadr pat))
                        (t nil))))
       (if rest
           `((,rest (subseq ,seq ,n)))
           (let ((p (car pat))
                 (rec (destruc (cdr pat) seq atom? (1+ n))))
             (if (funcall atom? p)
                 (cons `(,p (elt ,seq ,n))
                       rec)
                 (let ((var (gensym)))
                   (cons (cons `(,var (elt ,seq ,n))
                               (destruc p var atom?))
                         rec))))))))


(defmacro map-error-to-nil (&key form)
  "Call FORM and return the result.  If an error was thrown, return nil.
Note that the logging system is used by this Macro."
  
  (let ((err (gensym "err"))
	(result (gensym "result"))
	(return-value (gensym "value")))
    `(handler-case 
       (let ((,return-value ,form))
	 ,return-value)
       (condition (,err)
		  (log-message :warning
			       "Knowingly converting error to NIL: ~%~s ~%~s"
			       ,err
			       ',form)
		  nil)
       (:no-error (,result)
		  ,result))))


(defmacro with-corba-error ((var form condition &rest condition-options) &body body)
  `(let ((,var ,form))
    (unless ,var
      (log-message :error
		   "Could not create a ~A" ',var)
      (error (make-condition ',condition ,@condition-options)))
    ,@body))

(defmacro with-safe-binding ((var form condition &rest condition-options) &body body)
  `(let ((,var ,form))
    (unless ,var
      (error (make-condition ',condition ,@condition-options)))
    ,@body))


(defmacro deletef (item place &rest key-pairs)
  "Courtesy of Jim Meehan."
  (let ((item-var (gensym)))
    (multiple-value-bind (vars vals stores store-form access-form)
        (get-setf-expansion place)
      `(let* ((,item-var ,item)
              ,@(mapcar #'list vars vals)
              (,(car stores) (delete ,item-var ,access-form ,@key-pairs)))
         ,store-form))))

(defmacro multiplyf (place value)
  (multiple-value-bind (vars vals stores store-form access-form)
      (get-setf-expansion place)
    `(let* (,@(mapcar #'list vars vals)
	    (,(car stores) (* ,access-form ,value)))
      ,store-form)))


(defmacro swapf (place1 place2)
  (multiple-value-bind (vars1 vals1 stores1 store-form1 access-form1)
      (get-setf-expansion place1)
    (multiple-value-bind (vars2 vals2 stores2 store-form2 access-form2)
	(get-setf-expansion place2)
      `(,(if (null (cdr vars1)) 'let 'let*) (,@(mapcar #'list vars1 vals1))
	   (,(if (null (cdr vars2)) 'let 'let*) (,@(mapcar #'list vars2 vals2))
	       (let ((,(car stores1) ,access-form2)
		     (,(car stores2) ,access-form1))
		 ,store-form1
		 ,store-form2
		 nil))))))


(defmacro with-protected-side-effects (eval-form . cleanup-forms)
  "   This macro is similar to unwind-protect, except that the
   cleanup-forms get evaluated only if an error occurs.  That
   error is then re-thrown."
  (let ((err-sym (gensym "ERROR")))
    `(handler-case ,eval-form
       (condition (,err-sym)
	 ,@cleanup-forms
	 (error ,err-sym)))))


;;;
;;; use:
;;;   (with-functions-rebound ((old-fn-symbol1 new-fn-symbol1)
;;;                            (old-fn-symbol2 new-fn-symbol2)
;;;                            ...
;;;                            (old-fn-symboln new-fn-symboln))
;;;     . body)
;;;
;;
;;(defmacro with-functions-rebound ((&rest bindings) &body body)
;;  "Execute body where function values are redefined as per BINDINGS.
;;e.g. (WITH-FUNCTIONS-REBOUND  ((orig-foo new-foo)
;;                               (orig-bar new-bar))
;;        (do-stuff) ; calls to orig-foo are now calls to orig-bar
;;        ...)"
;;
;;  `(let ((original-functions
;;          (loop for (orig new) in ',bindings
;;                collect
;;                (let ((holder (gensym)))
;;                  (setf holder (symbol-function orig)
;;                        (symbol-function orig) (symbol-function new))
;;                  holder))))
;;    (unwind-protect (progn ,@body)
;;      (loop for (orig new) in ',bindings
;;            for orig-function  in original-functions
;;            do
;;            (setf (symbol-function orig) orig-function)))))

(defmacro with-functions-rebound ((&rest bindings) &body body)
  (let* ((unbound-fn-sym (gensym "UNBOUND-FUNCTION"))
	 (old-fn-names (mapcar #'first bindings))
	 (save-syms (mapcar #'(lambda (old-name) (gensym (symbol-name old-name))) old-fn-names)))
    `(let ((,unbound-fn-sym ',unbound-fn-sym))
       (let (,@(mapcar #'(lambda (save-sym old-fn-name)
			   `(,save-sym (if (fboundp ',old-fn-name)
					   (symbol-function ',old-fn-name)
					   ,unbound-fn-sym)))
		       save-syms old-fn-names))
	 (setf ,@(mapcan #'(lambda (binding)
			     `((symbol-function ',(first binding))
			       (symbol-function ',(second binding))))
			 bindings))
	 (unwind-protect (progn ,@body)
	   ,@(mapcar #'(lambda (save-sym old-fn-name)
			 `(if (eq ,save-sym ,unbound-fn-sym)
			   (fmakunbound ',old-fn-name)
			   (setf (symbol-function ',old-fn-name) ,save-sym)))
		     save-syms old-fn-names))))))

	
;;; This is a T style conditional.  It allows cond clauses of the form
;;; (cond ((foo x) => #'baz)
;;;       ...
;;;   )
;;; where the result of (foo x) is applied to the function baz if (foo x)
;;; returns a non-nil value.  Note that the function must be a function
;;; of one argument only, the result of the conditional in the cond clause.
;;;
(defmacro tcond (&rest clauses)
  (labels ((process-clauses (clauses count)
	     (if (null clauses) nil
	       (let ((clause (first clauses)))
		 (if (and (symbolp (second clause)) (string= (symbol-name (second clause)) "=>"))
		     (let ((result-arg (let ((*gensym-counter* count))
					 (gensym "RESULT-CLAUSE"))))
		       `(let ((,result-arg ,(first clause)))
			  (if ,result-arg
			      (funcall ,(third clause) ,result-arg)
			      ,(process-clauses (cdr clauses) (1+ count)))))
		     `(if ,(first clause)
		        ,(if (cddr clause)
			     `(progn ,@(rest clause))
			     (second clause))
		        ,(process-clauses (cdr clauses) count)))))))
    (process-clauses clauses 0)))


;;; GENERIC-MACROS.LISP ends here
